'use strict';

const fs = require('fs');
const glob = require('glob');
const md5 = require('md5');

const bundledEntries = {};
bundledEntries.getEntries = new Promise((resolve, reject) => {
    Object.defineProperty(bundledEntries, 'state', {
        enumerable: false,
        set: (newValue) => {
            resolve(newValue);
        }
    });
});

function removeMultilineComments(text, params) {
    let { discardComments } = params;
    return discardComments ? text.replace(/((\/\*)|(<!\-\-))(\s|\S){1,}?((\*\/)|(\-\->))/g, '') : text;
}

function getValidComment(searchText) {
    const matchQuotes = (searchText.match(/[']|["]|(\/){2}/g) || []);
    const commentsArray = (searchText.match(/(\/){2}(\s|\S)+/g) || [' //'])[0].split('//').slice(1);
    let commentIndex = -1;
    const COMMENT = '//';
    const quotes = {
        SINGLE: '\'',
        DOUBLE: '"',
        BACKTICK: '`',
        open: null
    };

    return matchQuotes.reduce((comment, item) => {
        commentIndex += item === COMMENT ? 1 : 0;

        if (item === COMMENT && !quotes.open) {
            return `//${commentsArray.slice(commentIndex).join('//')}`;
        } else if (!quotes.open) {
            quotes.open = [quotes.SINGLE, quotes.DOUBLE, quotes.BACKTICK].indexOf(item) > -1 ? item : quotes.open;
        } else if (item === quotes.open) {
            quotes.open = null;
        }

        return comment;
    }, '');
}

function removeSingleLineComments(text, compress) {

    const hasNoChars = text.match(/\S/) === null;

    if (text.indexOf('//') === -1) {
        return hasNoChars ? `${compress ? '' : '\n'}` : text;
    }

    const validComment = getValidComment(text, 0, '');
    return validComment ? text.replace(validComment, '') : text;
}

function singleLineParserChecks(line = '', params) {
    let output = line;
    let {compress, discardComments} = params;

    output = !discardComments && output || removeSingleLineComments(output, params.compress);
    output = !compress && output || output.replace(/[\f\t\v]+/g, '').replace(/(\s){2,}/g, ' ');

    return output;
}

function parseFileContents(fileContents, params) {
    let insideTemplateString = false;

    let output = removeMultilineComments(fileContents, params)
        .split(/[\n\r]/g)
        .map(line => {
            const matchBackTick = (line.match(/[`]|(\/){2}/g) || []);
            let hasValidComment = false;
            const backTickCount = matchBackTick.reduce((counter, item) => {
                counter += item === '`' ? 1 : 0;
                if (item === '//' && counter % 2 === 0 && !hasValidComment) {
                    hasValidComment = true;
                }
                return counter;
            }, 0);

            if (backTickCount % 2 > 0 && !hasValidComment) {
                /* Simple boolean toggle */
                insideTemplateString = !insideTemplateString;
            }
            if (insideTemplateString) {
                return line;
            }

            return singleLineParserChecks(line, params)
        })
        .join(params.compress ? '' : '\n')
        .replace(/[\n\r]{2,}/g, params.compress ? '' : '\n\n')
        .replace(/[ ]{1,}[\n\r]{1,}/g, '')
        .replace(params.compress ? /[\f\t\v\n\r]+/g : '', '')
        .replace(params.compress ? /(\s){2,}/g : ' ', ' ');

    return output;
}

function write(target, markup) {
    const pathArray = target.split('/');
    let path = `${pathArray.shift()}`;
    const recurse = () => {
        fs.lstat(path, err => {
            if(err && path !== '.' && path !== '') {
                try{
                    fs.mkdirSync(path);
                }
                catch(err) {}
            }

            if(pathArray[1]) {
                path = `${path}/${pathArray.shift()}`;
                recurse();
            }
            else {
                fs.writeFileSync(target, markup);
            }
        });
    };

    recurse();
}

function formatTemplateVars(string, templateVars) {
    return bundledEntries.getEntries.then(entries => {
        return Object.keys(templateVars).reduce((returnString, item) => {
            let returnValue = returnString;
            const stringValue = templateVars[item].toString();
            if (stringValue.indexOf('[hash]') > -1) {
                entries.forEach(entry => {
                    const hashReplacement = stringValue.replace('[hash]', entry.hash);
                    returnValue = returnValue.replace(`${entry.file}${item}`, `${entry.file}${hashReplacement}`);
                });
            } else {
                const variable = item.replace(/\?/g, '\\?');
                const regex = new RegExp(variable, 'g');
                returnValue = returnValue.replace(regex, templateVars[item]);
            }

            return returnValue;
        }, string);
    });
}

/**
 * @description Generate a compressed file at a target location, and replace any template literal variables present with a specific value
 * @param {(array|object)} params - The configuration for one or more sources and destinations
 * @param {(string|array)} params.sourceFiles - Either the list of files you need to minify and cache bust, or a globbing pattern
 * @param {string} params.sourceRoot - The root folder (lowest common folder) that the source files are located in
 * @param {string} params.targetRoot - The root folder (lowest common folder) that the processed files should be written to
 * @param {(object|function)} [params.templateVars={}] - An object (or function returning an object) that will contain the values associated with any template literal variables included in the source file
 * If you have the keyword '[hash]' as one of the templateVar values, then the md5 hash of the target file will replace the '[hash]' placeholder.
 * Example usage: templateVars: { '?cache=true': '?[hash]' }
 * @param {boolean} [params.compress=false] - Remove all extra white spaces, line feeds, tabs etc.
 * @param {boolean} [params.discardComments=false] - Remove all Javascript and HTML comments from the file
 * @param {boolean} [params.webpack=true] - Determines whether or not the method should be executed as a Webpack plugin. The '[hash]' replacement works only as a Webpack plugin
 * @returns {void}
 * */
function TemplateParser(params) {
    const options = (params instanceof Array) ? params : [params];

    options.forEach(properties => {

        const option = Object.assign({
            templateVars: {},
            compress: false,
            discardComments: false,
            webpack: true
        }, properties);

        if (option.webpack === false) {
            bundledEntries.state = [];
        }

        const files = (option.sourceFiles instanceof Array) ? option.sourceFiles : glob.sync(option.sourceFiles);

        files.forEach(file => {
            const fileContents = fs.readFileSync(file).toString();

            /* Detect what type of template variables property has been set */
            const templateVars = typeof option.templateVars === 'function' ? option.templateVars() : option.templateVars;

            formatTemplateVars(fileContents, templateVars).then(outputString => {
                write(file.replace(option.sourceRoot, option.targetRoot), parseFileContents(outputString, option));
            });
        });
    });
}

TemplateParser.prototype.apply = (compiler) => {

    compiler.plugin('done', (...opts) => {
        const assets = (opts.compilation) ? opts.compilation.assets : opts[0].compilation.assets;
        bundledEntries.state = Object.keys(assets).reduce((assetList, item) => {
            if (assets[item].children) {
                assetList.push({
                    file: item.split('/').pop(),
                    hash: md5(assets[item].children[0]._value)
                })
            }

            return assetList;
        }, []);
    });
};

module.exports = TemplateParser;