const sinon = require('sinon');
const fileParser = require('./file-copy-parse');
const fs = require('fs');
const path = require('path');

function cleanOutput() {
    fs.unlinkSync(`${process.env.PWD}/test-files/output/test.html`);
    fs.unlinkSync(`${process.env.PWD}/test-files/output/test.js`);
    fs.rmdirSync(`${process.env.PWD}/test-files/output`);
}

describe('FileParser', () => {
    //afterEach(cleanOutput);

    const htmlCompressResult = fs.readFileSync(`${process.env.PWD}/test-files/expected/test-compressed.html`).toString();
    const jsCompressResult = fs.readFileSync(`${process.env.PWD}/test-files/expected/test-compressed.js`).toString();
    const htmlNocommentAndCompressResult = fs.readFileSync(`${process.env.PWD}/test-files/expected/test-compressed.min.html`).toString();
    const jsNocommentAndCompressResult = fs.readFileSync(`${process.env.PWD}/test-files/expected/test-compressed.min.js`).toString();
    const htmlTemplateVarResult = fs.readFileSync(`${process.env.PWD}/test-files/expected/test-templatevars.html`).toString();

    it('Should create output files from the array of requests in the correct location', (done) => {
        fileParser([
            {
                sourceFiles: `${process.env.PWD}/test-files/test.html`,
                sourceRoot: `${process.env.PWD}/test-files`,
                targetRoot: `${process.env.PWD}/test-files/output`,
                webpack: false
            },
            {
                sourceFiles: `${process.env.PWD}/test-files/test.js`,
                sourceRoot: `${process.env.PWD}/test-files`,
                targetRoot: `${process.env.PWD}/test-files/output`,
                webpack: false
            }
        ]);

        const isFile = () => {
            try {
                const jsFile = fs.statSync(`${process.env.PWD}/test-files/output/test.js`);
                const htmlFile = fs.statSync(`${process.env.PWD}/test-files/output/test.html`);
                return jsFile.isFile() && htmlFile.isFile();
            } catch(err) {
                return false;
            }
        };

        setTimeout(function() {
            assert.isTrue(isFile(), 'Output files are created in test-files/output/');

            done();
        }, 1900);
    });

    it('Should create output files in the correct location', (done) => {
        fileParser({
            sourceFiles: [`${process.env.PWD}/test-files/test.html`, `${process.env.PWD}/test-files/test.js`],
            sourceRoot: `${process.env.PWD}/test-files`,
            targetRoot: `${process.env.PWD}/test-files/output`,
            webpack: false
        });

        const isFile = () => {
            try {
                const jsFile = fs.statSync(`${process.env.PWD}/test-files/output/test.js`);
                const htmlFile = fs.statSync(`${process.env.PWD}/test-files/output/test.html`);
                return jsFile.isFile() && htmlFile.isFile();
            } catch(err) {
                return false;
            }
        };

        setTimeout(function() {
            assert.isTrue(isFile(), 'Output files are created in test-files/output/');

            done();
        }, 1900);
    });

    it('Should compress a file by removing all irrelevant white space', (done) => {
        fileParser({
            sourceFiles: ['./test-files/test.html', './test-files/test.js'],
            sourceRoot: './test-files',
            targetRoot: './test-files/output',
            compress: true,
            webpack: false
        });

        const getFiles = () => {
            try {
                const jsFile = fs.readFileSync(`${process.env.PWD}/test-files/output/test.js`).toString();
                const htmlFile = fs.readFileSync(`${process.env.PWD}/test-files/output/test.html`).toString();
                return [jsFile, htmlFile];
            } catch(err) {
                return ['', ''];
            }
        };

        setTimeout(function() {
            const [jsFile, htmlFile] = getFiles();
            assert.strictEqual(jsFile, jsCompressResult, 'Javascript file is correctly compressed');
            assert.strictEqual(htmlFile, htmlCompressResult, 'HTML file is correctly compressed');

            done();
        }, 1900);
    });

    it('Should compress a file by removing all irrelevant white space and comments', (done) => {
        fileParser({
            sourceFiles: ['./test-files/test.html', './test-files/test.js'],
            sourceRoot: './test-files',
            targetRoot: './test-files/output',
            compress: true,
            discardComments: true,
            webpack: false
        });

        const getFiles = () => {
            try {
                const jsFile = fs.readFileSync(`${process.env.PWD}/test-files/output/test.js`).toString();
                const htmlFile = fs.readFileSync(`${process.env.PWD}/test-files/output/test.html`).toString();
                return [jsFile, htmlFile];
            } catch(err) {
                return ['', ''];
            }
        };

        setTimeout(function() {
            const [jsFile, htmlFile] = getFiles();
            assert.strictEqual(jsFile, jsNocommentAndCompressResult, 'Javascript file is correctly compressed and comments removed');
            assert.strictEqual(htmlFile, htmlNocommentAndCompressResult, 'HTML file is correctly compressed and comments removed');

            done();
        }, 1900);
    });

    it('Should compress the file and insert the required template variables', (done) => {
        fileParser({
            sourceFiles: ['./test-files/test.html', './test-files/test.js'],
            sourceRoot: './test-files',
            targetRoot: './test-files/output',
            compress: true,
            discardComments: true,
            templateVars: {
                '<!--vorlon-->': '<script src="http://vorlon.rtlxl.nl/vorlon.js"></script>',
                'cacheBuster=true': 1234567890987654321
            },
            webpack: false
        });

        const getFiles = () => {
            try {
                const jsFile = fs.readFileSync(`${process.env.PWD}/test-files/output/test.js`).toString();
                const htmlFile = fs.readFileSync(`${process.env.PWD}/test-files/output/test.html`).toString();
                return [jsFile, htmlFile];
            } catch(err) {
                return ['', ''];
            }
        };

        setTimeout(function() {
            const [jsFile, htmlFile] = getFiles();
            assert.strictEqual(jsFile, jsNocommentAndCompressResult, 'Javascript file is correctly compressed and comments removed');
            assert.strictEqual(htmlFile, htmlTemplateVarResult, 'HTML file is correctly compressed and comments removed');

            done();
        }, 1900);
    });

    it('Should compress the file and insert the required template variables from a function', (done) => {
        fileParser({
            sourceFiles: ['./test-files/test.html', './test-files/test.js'],
            sourceRoot: './test-files',
            targetRoot: './test-files/output',
            compress: true,
            discardComments: true,
            templateVars: () => ({
                '<!--vorlon-->': '<script src="http://vorlon.rtlxl.nl/vorlon.js"></script>',
                'cacheBuster=true': 1234567890987654321
            }),
            webpack: false
        });

        const getFiles = () => {
            try {
                const jsFile = fs.readFileSync(`${process.env.PWD}/test-files/output/test.js`).toString();
                const htmlFile = fs.readFileSync(`${process.env.PWD}/test-files/output/test.html`).toString();
                return [jsFile, htmlFile];
            } catch(err) {
                return ['', ''];
            }
        };

        setTimeout(function() {
            const [jsFile, htmlFile] = getFiles();
            assert.strictEqual(jsFile, jsNocommentAndCompressResult, 'Javascript file is correctly compressed and comments removed');
            assert.strictEqual(htmlFile, htmlTemplateVarResult, 'HTML file is correctly compressed and comments removed');

            done();
        }, 1900);
    });
});
